use clap::Parser;

/// program that generates ed25519 keypairs seeded by a passphrase and an optional ID.
#[derive(Parser)]
#[clap(
    name = "duralumin-keygen",
    version = "0.3.0",
    author = "No One <noonebtw@nirgendwo.xyz>"
)]
struct Opts {
    #[clap(short, long)]
    file: Option<String>,
}

fn main() -> anyhow::Result<()> {
    let opts = Opts::parse();
    println!("Generating ed25519 ssh keypair:");

    let desc = libduralumin::key_gen::cli::keygen_desc_from_stdin()?;

    let base_path = opts.file.unwrap_or_else(|| {
        if let Some(tag) = desc.tag.as_ref() {
            format!("duralumin_{}", tag.as_str())
        } else {
            "duralumin".to_owned()
        }
    });

    let keypair = libduralumin::key_gen::generate_key(desc)?;

    println!(
        "Your key fingerprint is: Sha256:{}",
        keypair.fingerprint_base64()
    );

    println!(
        "RandomArt:\n{}",
        keypair.randomart().render("ED25519 256", "SHA256")?
    );

    let private_path = base_path.clone();
    let public_path = base_path.clone() + ".pub";

    let (private_key, public_key) = keypair.encode_keys()?;
    std::fs::write(&private_path, private_key)?;

    std::fs::write(&public_path, public_key)?;

    #[cfg(target_family = "unix")]
    {
        use std::fs::Permissions;
        use std::os::unix::fs::PermissionsExt;

        std::fs::set_permissions(private_path, Permissions::from_mode(0o0600))?;
        std::fs::set_permissions(public_path, Permissions::from_mode(0o0600))?;
    }

    Ok(())
}
